
/*************************************************************************

SIUDS - Secure Interface for a Universal Data Server.
Copyright (C) 2000  Flavia Linhalis, Dilvan Moreira

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact: flavialin@bol.com.br
         dilvan@computer.org
         Paper mail: SCE-ICMC-USP Caixa Postal 668
                     Av Dr Carlos Botelho, 1465
                     13560-970 Sao Carlos-SP
                     BRAZIL

***************************************************************************/

package gateway;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.net.Socket;
import java.net.InetAddress;

import pool.Pool;

/**
 *  Its a class of the Gateway agent. It permits a file transfer.
 */
class Server {
    private String poolPath = "c:\\siuds\\";
    private Socket socket;
    private BufferedReader in;
    private PrintWriter out;
    private static final int BUFSIZ = 1024;

    CertificateVerifier cerVer;

    /**
     * Creates the two way communication for this socket
     * @param  s  the Socket
     */
    public Server(Socket s) throws Exception {
	     socket = s;
	     in = new BufferedReader(
		   new InputStreamReader(socket.getInputStream()));

	     out = new PrintWriter(
		              new BufferedWriter(
		                  new OutputStreamWriter(socket.getOutputStream())),true);
    }

   /**
    * Transfers the file to the pool of agents directory.
    * After the transfer is complete the Jar verification begins.
    * @see JarVerifier
    * @see CertificateVerifier
    * @return <code>true</code> if the Jar and certificate verification succeed, <code>false</code> otherwise.
    */
   public boolean verifyAll() throws Exception {
	   boolean allValid = false;
 	   try {
       while (true) {
		      /* Reads file name from the client */
		      String str = null;
		      str = in.readLine();
		      String path = poolPath + str;

		      /* Creates file name on the specified path*/
		      File file = new File(path);
		      if (file.exists()) {
			       out.println("File already exists");
			       break;
		      }
		      out.println("Opening data connection.");

		      /* Gets the bytes from the client and writes to the new file*/
		      InputStream in2 = socket.getInputStream();
		      BufferedOutputStream out2 = new BufferedOutputStream(
                                          new FileOutputStream(path));
 		      byte buf[] = new byte[BUFSIZ];
		      int nread;
	  	    while ((nread = in2.read(buf, 0, BUFSIZ)) > 0)
			    out2.write(buf, 0, nread);
		      in2.close();
		      out2.flush(); /*Closes the file*/
		      out2.close();

        	/* Jar verification - Calls JarVerifier*/
		      JarVerifier jv = new JarVerifier();
		      boolean validJar = jv.verification(str);
		      if (validJar) {

		          /* Calls CerfificateVerifier */
		          String jarName = str;
		          boolean ValidCert = cerVer.getCert(jarName);
		          if (ValidCert) {
			          allValid = true;
		          }
		      }
	     }
	   } catch (Exception e) {
        e.printStackTrace();
	   } finally {
	       try {
		        socket.close();
	       } catch (Exception e) { }
     }
   	 return allValid;
   }
/*
 * Called when the certificate is ready to be verified.
 */
   void enviaCert(CertificateVerifier cvR) {
	    cerVer = cvR;
   }
}
