
/*************************************************************************

SIUDS - Secure Interface for a Universal Data Server. 
Copyright (C) 2000  Flavia Linhalis, Dilvan Moreira

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact: flavialin@bol.com.br
         dilvan@computer.org
         Paper mail: SCE-ICMC-USP Caixa Postal 668
                     Av Dr Carlos Botelho, 1465
                     13560-970 Sao Carlos-SP
                     BRAZIL

***************************************************************************/

package pool;

import structures.DBImplementation;

/** 
* Controlls the jar lifetime according to the time associated to 
* its group.
*/
public class TimeCounter extends Thread {
	DBImplementation dbi = new DBImplementation();
	JarRunner jar;
	long time = 0;

/** 
* Controlls the jar lifetime according to the time associated to
* its group.
* The lifetime of a group have to be stored in the database.
* @see DBImplemtentation
* @param String the group whitch lifetime will be controlled
* @exception LifetimeException
*             Thrown when the Lifetime of the agent ends.
* @exception InterruptedException
*            If this thread is interrupted by another thread
*/
	public TimeCounter(String groupName) {
	    time = dbi.getTime(groupName);
	}

	public void run() {
	    System.out.println("Tempo: " + time);
	    if (time == 0) {
		  System.out.println("This jar can run indefinetly.");
	    } else {
		       System.out.println("This jar has " + time + " milliseconds to run.");
           try {
			        sleep(time);
		       } catch (InterruptedException e) {
		          e.printStackTrace();
		       }
		       try {
		          throw new LifetimeException("Jar lifetime has expired");
		       } catch (LifetimeException le) {
		          le.printStackTrace();
		       }
           String threadName = Thread.currentThread().getName();
		       jar.stop();
		       dbi.removeAssociation(threadName);
	    }
	}

/*
* Called when the jar is ready to run and its lifetime is ready
* to be counted.
*/
	void sendJar(JarRunner jarR) {
		jar = jarR;
	}
}