
/*************************************************************************

SIUDS - Secure Interface for a Universal Data Server. 
Copyright (C) 2000  Flavia Linhalis, Dilvan Moreira

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Contact: flavialin@bol.com.br
         dilvan@computer.org
         Paper mail: SCE-ICMC-USP Caixa Postal 668
                     Av Dr Carlos Botelho, 1465
                     13560-970 Sao Carlos-SP
                     BRAZIL

***************************************************************************/

package structures;

import java.io.Serializable;
import java.util.Hashtable;

/**
 * Sets in the Hashtable and gets back from the Hashtable, information about the threads and their associated groups and agents.
 * @see Hashtable
 */
public class StoreThreadsControll implements Serializable {

    private Hashtable threadsDataBase = new Hashtable();

   /**
    * Adds a new thread of excecution and its assiciated group and agent to the <code>Hashtable</code>.
    * @param threadName  the thread name. It is the <code>Hashtable</code> entry key.
    * @param groupName  the group name
    * @param agentName  the agent name
    */
    void setThreadsDataBase(String threadName, String groupName, String agentName) {
	     String s[] = {groupName, agentName};
       threadsDataBase.put(threadName, s);
    }

   /**
    * Gets the group corresponding to this thread.
    * @param threadName  the thread name
    * @return the group name
    */
    String getGroupThread(String threadName) {
        String s[] = (String[])threadsDataBase.get(threadName);
        return s[0];
    }

   /**
    * Gets the agent corresponding to this thread.
    * @param threadName  the thread name
    * @return the agent name
    */
    public String getAgentThread(String threadName) {
        String s[] = (String[])threadsDataBase.get(threadName);
        return s[1];
    }

   /**
    * Gets the <code>Hashtable</code> containing all the threads and associated groups and agents objects.
    * @return the <code>Hashtable</code> containing the threads database.
    */
    public Hashtable getThreadsDataBase() {
	     return threadsDataBase;
    }
}